########################### PSQF 6270 Example 4a using R #################################

# Set width of output and number of significant digits printed,
# number of digits before using scientific notation, shut off significance stars
options(width=120, digits=8, scipen=9, show.signif.stars=FALSE)

#####  Check to see if packages are downloaded, install if not, then load  #####

if (!require("haven")) install.packages("haven")
library(haven) # To import SAS data with labels as table

if (!require("expss")) install.packages("expss")
library(expss) # To add variable and value labels, sorting

if (!require("TeachingDemos")) install.packages("TeachingDemos")
library(TeachingDemos) # To create text output files

if (!require("psych")) install.packages("psych")
library(psych) # To add descriptive summary functions

if (!require("multcomp")) install.packages("multcomp")
library(multcomp) # To do glht linear combinations in GLMs

if (!require("VGAM")) install.packages("VGAM")
library(VGAM) # To fit lots of binomial (or count) models

if (!require("lme4")) install.packages("lme4")
library(lme4) # To fit models with random effects


######################################################################################
#####                  BEGIN DATA MANIPULATION OF EXAMPLE 4a                     #####
#####                   CHANGE "filesave" TO YOUR FOLDER PATH                    #####
######################################################################################

# Define variables for working directory and data name
filesave = "C:\\Dropbox/23_PSQF6270/PSQF6270_Example4a/"
filename = "PSQF6270_Example4a.sas7bdat"
setwd(dir=filesave)

# Import Example 4a SAS data
Example4a = read_sas(data_file=paste0(filesave,filename)) 
# Convert to data frame without labels to use for analysis
Example4a = as.data.frame(Example4a)

# Label existing variables for analysis
#NLIvSLI=     "Group: 0=NLI, 1=SLI" 
#momed12=     "Mother Education (0=12 years")
#PropCorrect= "Grammar Test Proportion Correct"

# Create number correct for denominator of binomial outcome
Example4a$Ntrials=100
Example4a$Ncorrect=Example4a$PropCorrect*Example4a$Ntrials
Example4a$Ncorrect=round(Example4a$Ncorrect,0)
# Compute number incorrect for zero-inflated binomial model
Example4a$Nincorrect=Example4a$Ntrials-Example4a$Ncorrect
Example4a$PropIncorrect=1-Example4a$PropCorrect


######################################################################################
#####                    BEGIN EXAMPLE 4a BINOMIAL MODELS                        #####
######################################################################################

# Open external file to save results to
txtStart(file=paste0(filesave,"PSQF6270_Example4a_R_Output.txt"))

print("R Distribution of Proportion Correct")
describe(x=Example4a$PropCorrect)

# To save a plot: open a file, create the plot, then close the file
png(file = "R Proportion Correct Histogram.png")  # open file
hist(x=Example4a$PropCorrect, freq=FALSE,
     ylab="Density",xlab="Grammar Test Proportion Correct") # axis labels
dev.off()  # close file

txtComment(" ") # insert blank space in output 

# Save sample size as variable for use in Pearson chi-square/DF fit
N=97

############################################################################################
print("R Empty Binomial Model using vglm and two outcome columns")
ModelEmpty = vglm(data=Example4a, binomialff(link="logitlink", multiple.responses=FALSE), 
                  formula=cbind(Ncorrect,Nincorrect)~1) # Can also use multiv format of 0/1
print("Print -2LL with results"); -2*logLik(ModelEmpty); summary(ModelEmpty)

print("Get intercept in probability")
ModelEmptyProb=1/(1+exp(-1*coefficients(ModelEmpty)));ModelEmptyProb 

print("Pearson Chi-Square / DF Index of Fit")
sum(residuals(ModelEmpty, type="pearson")^2)/(N-1)  # STATA N-k

txtComment(" ") # insert blank space in output 

############################################################################################
print("R Two-Predictor Binomial Model using vglm")
ModelBin = vglm(data=Example4a, binomialff(link="logitlink", multiple.responses=FALSE), 
                formula=cbind(Ncorrect,Nincorrect)~1+NLIvSLI+momed12)
print("Print -2LL, AIC, BIC, and results")
-2*logLik(ModelBin); AIC(ModelBin); BIC(ModelBin); summary(ModelBin) 

print("Pearson Chi-Square / DF Index of Fit")
sum(residuals(ModelBin, type="pearson")^2)/(N-3)  # STATA N-k

print("Multiv Wald test of model")
BinTestR2 = glht(model=ModelBin, linfct=c("NLIvSLI=0","momed12=0"))
print(summary(BinTestR2, test=Chisqtest()), digits="8") # Joint chi-square test

print("Get odds ratios with 95% CIs")
exp(cbind(OR=coefficients(ModelBin), confint.default(ModelBin)))

txtComment(" ") # insert blank space in output

############################################################################################
print("R Two-Predictor Binomial Model using glmer")
print("Additive Overdispersion via Random Intercept Variance (1|ID)")
print("Also known as observation-level random effect for overdispersion")
ModelBinAdd = glmer(data=Example4a, family=binomial(link="logit"), 
                    formula=cbind(Ncorrect,Nincorrect)~1+NLIvSLI+momed12+(1|ID))
print("Print -2LL, AIC, BIC, and results")
-2*logLik(ModelBinAdd); AIC(ModelBinAdd); BIC(ModelBinAdd); summary(ModelBinAdd) 

print("Pearson Chi-Square / DF Index of Fit")
sum(residuals(ModelBinAdd, type="pearson")^2)/(N-3)  # STATA N-k

print("Multiv Wald test of model")
BinAddTestR2 = glht(model=ModelBinAdd, linfct=c("NLIvSLI=0","momed12=0"))
print(summary(BinAddTestR2, test=Chisqtest()), digits="8") # Joint chi-square test 

print("Likelihood Ratio Test for Addition of Random Intercept Variance")
DevTestA=-2*(logLik(ModelBin)-logLik(ModelBinAdd))
RegPvalueA=pchisq((DevTestA), df=1, lower.tail=FALSE)
MixPvalueA=RegPvalueA/2
print("Test Statistic, Regular and Mixture P-values for DF=1") 
DevTestA; RegPvalueA; MixPvalueA

print("Get odds ratios with 95% CIs")
cbind(OR=exp(ModelBinAdd@beta), exp(confint(ModelBinAdd, parm="beta_", method="Wald")))

txtComment(" ") # insert blank space in output

############################################################################################
print("R Two-Predictor Binomial Model using vglm with Multiplicative Overdispersion")
ModelBetaBin = vglm(data=Example4a, betabinomial(lmu="logitlink", lrho="logitlink"), 
                    formula=cbind(Ncorrect,Nincorrect)~1+NLIvSLI+momed12)
print("Print -2LL, AIC, BIC, and results")
-2*logLik(ModelBetaBin); AIC(ModelBetaBin); BIC(ModelBetaBin); summary(ModelBetaBin) 

print("Pearson Chi-Square / DF Index of Fit")
sum(residuals(ModelBetaBin, type="pearson")^2)/(N-3)

print("Multiv Wald test of model")
BinMultTestR2 = glht(model=ModelBetaBin, linfct=c("NLIvSLI=0","momed12=0"))
print(summary(BinMultTestR2, test=Chisqtest()), digits="8") # Joint chi-square test 

print("Likelihood Ratio Test for Addition of Multiplicative Overdispersion")
DevTestB=-2*(logLik(ModelBin)-logLik(ModelBetaBin))
RegPvalueB=pchisq((DevTestB), df=1, lower.tail=FALSE); MixPvalueB=RegPvalueB/2
print("Test Statistic, Regular and Mixture P-values for DF=1") 
DevTestB; RegPvalueB; MixPvalueB

print("Save predicted propcorrect and correlate with actual propcorrect") 
Example4a$PredBetaBin = predict(ModelBetaBin, type="response")
rPredBetaBin = cor.test(Example4a$PredBetaBin, Example4a$PropCorrect, method="pearson")
print("R2"); rPredBetaBin$estimate^2

print("Get odds ratios with 95% CIs")
exp(cbind(OR=coefficients(ModelBetaBin), confint.default(ModelBetaBin)))

txtComment(" ") # insert blank space in output

############################################################################################
print("R Two-Predictor Zero-Inflated Binomial Model using vglm Predicting Nincorrect")
ModelZBin = vglm(data=Example4a, zibinomialff(lprob="logitlink", lonempstr0="logitlink",
                 multiple.responses=FALSE, ionempstr0=NULL, zero="onempstr0"), 
                 formula=cbind(Nincorrect,Ncorrect)~1+NLIvSLI+momed12)
print("Print -2LL, AIC, BIC, and results") 
-2*logLik(ModelZBin); AIC(ModelZBin); BIC(ModelZBin); summary(ModelZBin) 

print("Pearson Chi-Square / DF Index of Fit")
sum(residuals(ModelZBin, type="pearson")^2)/(N-3)

print("Multiv Wald test of model")
ZBinMultTestR2 = glht(model=ModelZBin, linfct=c("NLIvSLI=0","momed12=0"))
print(summary(ZBinMultTestR2, test=Chisqtest()), digits="8") # Joint chi-square test 

print("Likelihood Ratio Test for Addition of Zero-Inflation over Binomial")
DevTestC=-2*(logLik(ModelBin)-logLik(ModelZBin))
RegPvalueC=pchisq((DevTestC), df=1, lower.tail=FALSE)
MixPvalueC=RegPvalueC/2
print("Test Statistic, Regular and Mixture P-values for DF=1") 
DevTestC; RegPvalueC; MixPvalueC

print("Get odds ratios with 95% CIs")
exp(cbind(OR=coefficients(ModelZBin), confint.default(ModelZBin)))

txtComment(" ") # insert blank space in output

# I could not find zero-inflated beta-binomial regression in R, so I gave up

############################################################################################
# Close external output file
txtStop()


