// Prevent "more" messages from appearing
set more off
// Control line length
set linesize 150

********************************************************************************
*******             BEGIN DATA MANIPULATION FOR EXAMPLE 4a               *******
*******             CHANGE "filesave" to your folder path                *******
********************************************************************************

// Defining global variable for file location to be replaced in code below
// \\Client\ precedes path in Virtual Desktop outside H drive;
global filesave "C:\Dropbox\23_PSQF6270\PSQF6270_Example4a"

// Import Example 4a Stata data
use "$filesave\PSQF6270_Example4a.dta", clear

// Label existing vriables for analysis
label variable nlivsli     "Group: 0=NLI, 1=SLI"
label variable momed12     "Mother Education (0=12 years)"
label variable propcorrect "Grammar Test Proportion Correct"
// Create number correct for denominator of binomial outcome
gen ntrials=100
gen ncorrect=round(propcorrect*ntrials,1)
// Compute number incorrect for zero-inflated binomial model
gen nincorrect=ntrials-ncorrect
gen propincorrect=1-propcorrect

// Find betabin and zbin, install before continuing
// search betabin
// search zbin
 
********************************************************************************
******               BEGIN EXAMPLE 4a BINOMIAL MODELS                    *******
********************************************************************************

// Open external file to save results to
log using $filesave\PSQF6270_Example4a_STATA_Output.log, replace name(Example4a)
 
display "STATA Distribution of Proportion Correct"
summarize propcorrect
hist propcorrect, percent start(0) width(.05) 
graph export "$filesave\STATA Proportion Correct Histogram.png", replace

display "STATA Empty Binomial Model using glm -- ntrials is denominator"
glm ncorrect, link(logit) family(binomial ntrials)
display "-2LL= " e(ll)*-2  // Print -2LL for model  
margins // Get intercept in # expected events (ILINK*Ntrials = percent here)


display "STATA Two-Predictor Binomial Model using glm" 
glm ncorrect c.nlivsli c.momed12, link(logit) family(binomial ntrials) nolog
display "-2LL= " e(ll)*-2  // Print -2LL for model  
estat ic, n(97)            // Print AIC and BIC
test (c.nlivsli=0)(c.momed12=0)  // Multiv Wald test of model

display "STATA Two-Predictor Binomial Model -- get odds ratios using eform" 
glm ncorrect c.nlivsli c.momed12, link(logit) family(binomial ntrials) eform  


display "STATA Two-Predictor Binomial Model with Additive Overdispersion" 
display "Using meglm instead; || id. adds random intercept variance"
display "Also known as observation-level random effect for overdispersion"
meglm ncorrect c.nlivsli c.momed12, || id: , /// 
      link(logit) family(binomial ntrials) intmethod(laplace) nolog
display "-2LL= " e(ll)*-2  // Print -2LL for model
estat ic, n(97)            // Print AIC and BIC
// LRT for added random intercept variance is done for you automatically
// Multiv Wald test of model also given in meglm for you
// Pearson chi-square/DF not given

display "STATA Two-Predictor Binomial Model with Additive Overdispersion" 
display "Using meglm instead and getting odds ratios using eform"
meglm ncorrect c.nlivsli c.momed12, || id: , /// 
      link(logit) family(binomial ntrials) intmethod(laplace) eform nolog


display "STATA Two-Predictor Beta-Binomial Model with Multiplicative Overdispersion" 
display "Using betabin instead that has beta-binomial distribution"
betabin ncorrect c.nlivsli c.momed12, link(logit) n(ntrials) nolog
// LRT for multiplicative overdispersion is done for you automatically 
// Pearson chi-square/DF fit not given 
display "-2LL= " e(ll)*-2  // Print -2LL for model  
estat ic, n(97)            // Print AIC and BIC
test (c.nlivsli=0)(c.momed12=0)  // Multiv Wald test of model
predict yhatBBpred // Save predicted propcorrect per real person to dataset
corr yhatBBpred propcorrect  // Get corr of pred and actual propcorrect
display "R2= " r(rho)^2      // Print R2 relative to empty model

display "STATA Two-Predictor Binomial Model with Multiplicative Overdispersion" 
display "Using betabin and Getting Odds Ratios using eform"
betabin ncorrect c.nlivsli c.momed12, link(logit) n(ntrials) eform nolog


display "STATA Two-Predictor Zero-Inflated Binomial Model" 
display "Use zbin and predict nincorrect instead"
display "ilink is for submodel predicting extra 0 (empty here)"
zib nincorrect c.nlivsli c.momed12, ///
    link(logit) n(ntrials) ilink(logit) inflate(_cons) nolog
// Pearson chi-square/DF fit not given 
display "-2LL= " e(ll)*-2  // Print -2LL for model
estat ic, n(97)            // Print AIC and BIC
test (c.nlivsli=0)(c.momed12=0)  // Multiv Wald test of model
display "LRT for zero-inflation (binomial vs zero-inflated binomial)"
display "Test Statistic (df=1)= "          1531.7332-988.2183
display "Regular p-value= "     (1-chi2(1, 1531.7332-988.2183))
display "Mixture p-value= " 0.5*(1-chi2(1, 1531.7332-988.2183))


display "STATA Two-Predictor Zero-Inflated Binomial Model" 
display "Use zbin and predict nincorrect instead, get odds ratios"
display "ilink is for submodel predicting extra 0 (empty here)"
zib nincorrect c.nlivsli c.momed12, ///
    link(logit) n(ntrials) ilink(logit) inflate(_cons) eform nolog
	 

display "STATA Two-Predictor Zero-Inflated Beta-Binomial Model" 
display "Use zibbin and predict nincorrect instead"
zibbin nincorrect c.nlivsli c.momed12, ///
       link(logit) n(ntrials) ilink(logit) inflate(_cons) nolog
// Pearson chi-square/DF fit not given 
display "-2LL= " e(ll)*-2  // Print -2LL for model
estat ic, n(97)            // Print AIC and BIC
test (c.nlivsli=0)(c.momed12=0)  // Multiv Wald test of model
display "LRT for overdispersion (zero-inflated: binomial vs beta-binomial)"
display "Test Statistic (df=1)= "          988.2183-527.5780
display "Regular p-value= "     (1-chi2(1, 988.2183-527.5780))
display "Mixture p-value= " 0.5*(1-chi2(1, 988.2183-527.5780))
// Save predicted propincorrect per real person to dataset
predict yhatZIBB, xb             // Predicted outcome in logits
gen Npred=1/(1+exp(-1*yhatZIBB)) // Convert to probability
corr Npred propincorrect         // Get corr of pred and actual propincorrect
display "R2=" r(rho)^2           // Print R2 relative to empty model

display "STATA Two-Predictor Zero-Inflated Beta-Binomial Model" 
display "Use zibbin and predict nincorrect instead, get odds ratios"
zibbin nincorrect c.nlivsli c.momed12, ///
       link(logit) n(ntrials) ilink(logit) inflate(_cons) eform nolog
		 

display "STATA Two-Predictor Zero-Inflated Beta-Binomial Model" 
display "Switch to zibbin and predict nincorrect instead"
display "Add two predictors of being extra zero"
zibbin nincorrect c.nlivsli c.momed12, link(logit) n(ntrials) ///
                  ilink(logit) inflate(c.nlivsli c.momed12) nolog
display "-2LL= " e(ll)*-2  // Print -2LL for model

display "STATA Two-Predictor Zero-Inflated Beta-Binomial Model" 
display "Switch to zibbin and predict nincorrect instead, get odds ratios"
display "Add two predictors of being extra zero"
zibbin nincorrect c.nlivsli c.momed12, link(logit) n(ntrials) ///
                  ilink(logit) inflate(c.nlivsli c.momed12) eform nolog


// Close external file   
log close Example4a
